@@ -41,7 +41,7 @@ proc include { filename } {
 namespace eval rivet {}
 namespace eval rivet {
 	proc ::rivet::reset {} {
-		unset -nocomplain ::rivet::header_pairs ::rivet::statuscode ::rivet::header_redirect ::rivet::cache_vars ::rivet::cache_vars_qs ::rivet::cache_vars_post ::rivet::cache_vars_contenttype ::rivet::cache_vars_contenttype_var ::rivet::cache_tmpdir ::rivet::transfer_encoding ::rivet::sent_final_chunk
+		unset -nocomplain ::rivet::header_pairs ::rivet::statuscode ::rivet::header_redirect ::rivet::cache_vars ::rivet::cache_vars_qs ::rivet::cache_vars_post ::rivet::cache_vars_contenttype ::rivet::cache_vars_contenttype_var ::rivet::cache_tmpdir ::rivet::transfer_encoding ::rivet::sent_final_chunk ::rivet::connection
 
 		if {[info exists ::rivet::cache_uploads]} {
 			foreach {var namefd} [array get ::rivet::cache_uploads] {
@@ -56,7 +56,7 @@ namespace eval rivet {
 		}
 
 		array set ::rivet::header_pairs {}
-		set ::rivet::header_type "text/html"
+		set ::rivet::header_pairs(content-type) "text/html"
 		set ::rivet::header_sent 0
 		set ::rivet::output_buffer ""
 		set ::rivet::send_no_content 0
@@ -127,7 +127,12 @@ namespace eval rivet {
 	::rivet::reset
 }
 
-proc rivet_flush {{final_flush 0}} {
+proc rivet_flush args {
+	set final_flush 0
+	if {[lsearch -exact $args "-final"] != "-1"} {
+		set final_flush 1
+	}
+
 	set outchan stdout
 	if {[info exists ::env(RIVET_INTERFACE)]} {
 		set outchan [lindex $::env(RIVET_INTERFACE) 2]
@@ -143,18 +148,6 @@ proc rivet_flush {{final_flush 0}} {
 
 		::rivet::cgi_server_writehttpheader $::rivet::statuscode
 
-		if {![info exists ::rivet::header_redirect]} {
-			tcl_puts $outchan "Content-type: $::rivet::header_type"
-			foreach {var val} [array get ::rivet::header_pairs] {
-				tcl_puts $outchan "$var: $val"
-			}
-		} else {
-			tcl_puts $outchan "Location: $::rivet::header_redirect"
-			tcl_puts $outchan ""
-			abort_page
-		}
-		tcl_puts $outchan ""
-
 		unset -nocomplain ::rivet::statuscode ::rivet::header_redirect ::rivet::header_pairs
 	}
 
@@ -177,8 +170,8 @@ proc rivet_flush {{final_flush 0}} {
 		}
 	}
 
-	if {[info exists ::rivet::transfer_encoding] && $::rivet::transfer_encoding == "chunked" && ![info exists ::rivet::sent_final_chunk]} {
-		if {$final_flush == "1"} {
+	if {$final_flush == "1"} {
+		if {[info exists ::rivet::transfer_encoding] && $::rivet::transfer_encoding == "chunked" && ![info exists ::rivet::sent_final_chunk]} {
 			fconfigure $outchan -translation "crlf"
 
 			tcl_puts $outchan "0"
@@ -242,12 +235,13 @@ proc rivet_error {} {
 
 	if {!$::rivet::header_sent} {
 		set ::rivet::header_sent 1
-		::rivet::cgi_server_writehttpheader 200 [string length $errmsg]
-		tcl_puts $outchan "Content-type: text/html"
-		tcl_puts $outchan ""
-		tcl_puts -nonewline $outchan $errmsg
+
+		headers type "text/html"
+
+		::rivet::cgi_server_writehttpheader 200 "" [string length $errmsg]
 	}
 
+	tcl_puts -nonewline $outchan $errmsg
 }
 
 proc rivet_puts args {
@@ -278,27 +272,9 @@ proc rivet_puts args {
 		}
 	} else {
 		if {$fd == "stdout"} {
-			set str [lindex $args 0]$appendchar
-			set strlen [string length $str]
+			append ::rivet::output_buffer [lindex $args 0]$appendchar
 
-			if {!$::rivet::send_no_content && $strlen != "0"} {
-				if {[info exists ::rivet::transfer_encoding] && $::rivet::transfer_encoding == "chunked"} {
-					fconfigure $outchan -translation "crlf"
-
-					tcl_puts $outchan [format %x $strlen]
-				}
-
-				fconfigure $outchan -translation binary
-				tcl_puts -nonewline $outchan $str
-
-				if {[info exists ::rivet::transfer_encoding] && $::rivet::transfer_encoding == "chunked"} {
-					fconfigure $outchan -translation "crlf"
-
-					tcl_puts $outchan ""
-
-					fconfigure $outchan -translation binary
-				}
-			}
+			rivet_flush
 		} else {
 			tcl_puts -nonewline $fd [lindex $args 0]$appendchar
 		}
@@ -735,18 +711,18 @@ proc headers args {
 	set cmd [lindex $args 0]
 	switch -- $cmd {
 		"set" {
-			set var [lindex $args 1]
+			set var [string tolower [lindex $args 1]]
 			set val [lindex $args 2]
 			set ::rivet::header_pairs($var) $val
 		}
 		"add" {
-			set var [lindex $args 1]
+			set var [string tolower [lindex $args 1]]
 			set val [lindex $args 2]
 			append ::rivet::header_pairs($var) $val
 		}
 		"type" {
 			set val [lindex $args 1]
-			set ::rivet::header_type $val
+			set ::rivet::header_pairs(content-type) $val
 		}
 		"redirect" {
 			set val [lindex $args 1]
@@ -807,30 +783,56 @@ proc ::rivet::cgi_server_writehttpheader {statuscode {useenv ""} {length -1}} {
 
 			unset -nocomplain ::rivet::transfer_encoding
 
+			if {[info exists ::rivet::header_pairs(content-length)]} {
+				set out_contentlength $::rivet::header_pairs(content-length)
+
+				if {$length == "-1"} {
+					set length $out_contentlength
+				}
+			}
+
 			if {$headers(CONNECTION) == "keep-alive"} {
 				if {$length != -1} {
-					tcl_puts $outchan "Content-Length: $length"
-					tcl_puts $outchan "Connection: keep-alive"
+					set ::rivet::header_pairs(content-length) $length
+					set ::rivet::header_pairs(connection) "keep-alive"
+
+					set ::rivet::connection "keep-alive"
 				} else {
 					if {$statuscode == "200"} {
-						tcl_puts $outchan "Transfer-Encoding: chunked"
-						tcl_puts $outchan "Connection: keep-alive"
+						set ::rivet::header_pairs(transfer-encoding) "chunked"
+						set ::rivet::header_pairs(connection) "keep-alive"
+
 						set ::rivet::transfer_encoding "chunked"
+						set ::rivet::connection "keep-alive"
 					} else {
-						tcl_puts $outchan "Connection: close"
+						set ::rivet::header_pairs(connection) "close"
+
+						set ::rivet::connection "close"
 					}
 				}
 			} else {
-				tcl_puts $outchan "Connection: close"
+				set ::rivet::header_pairs(connection) "close"
+
+				set ::rivet::connection "close"
 			}
 
 			fconfigure $outchan -translation binary
-
-			return
 		}
+	} else {
+		tcl_puts $outchan "Status: $statuscode [::rivet::statuscode_to_str $statuscode]"
 	}
 
-	tcl_puts $outchan "Status: $statuscode [::rivet::statuscode_to_str $statuscode]"
+	if {![info exists ::rivet::header_redirect]} {
+		foreach {var val} [array get ::rivet::header_pairs] {
+			tcl_puts $outchan "$var: $val"
+		}
+	} else {
+		tcl_puts $outchan "Location: $::rivet::header_redirect"
+		tcl_puts $outchan ""
+		abort_page
+	}
+
+	tcl_puts $outchan ""
 }
 
 proc load_headers args { }
