@@ -21,7 +21,6 @@ proc call_page {{useenv ""} {createinterp 0}} {
 		set inchan [lindex $env(RIVET_INTERFACE) 1]
 		set outchan [lindex $env(RIVET_INTERFACE) 2]
 		set elogchan [lindex $env(RIVET_INTERFACE) 3]
-		array set headers [lindex $env(RIVET_INTERFACE) 4]
 	}
 
 	# Determine if a sub-file has been requested
@@ -35,12 +34,7 @@ proc call_page {{useenv ""} {createinterp 0}} {
 	if {[info exists env(PATH_INFO)]} {
 		set targetfile "$::starkit::topdir/$env(PATH_INFO)"
 	} else {
-		foreach chk_indexfile $indexfiles {
-			set targetfile [file join $::starkit::topdir $chk_indexfile]
-			if {[file exists $targetfile]} {
-				break
-			}
-		}
+		set targetfile [file join $::starkit::topdir]
 	}
 	
 	# If the file specified is a directory, look for an index
@@ -89,20 +83,14 @@ proc call_page {{useenv ""} {createinterp 0}} {
 		if {$targetfile == "__RIVETSTARKIT_FORBIDDEN__"} {
 			# Return a 403 (Forbidden)
 			::rivet::cgi_server_writehttpheader 403 [array get env]
-			tcl_puts $outchan "Content-type: text/html"
-			tcl_puts $outchan ""
 			tcl_puts $outchan "<html><head><title>Forbidden</title></head><body><h1>File Access Forbidden</h1></body>"
 		} elseif {[file tail $targetfile] == "__RIVETSTARKIT_INDEX__"} {
 			# Return a 403 (Forbidden)
 			::rivet::cgi_server_writehttpheader 403 [array get env]
-			tcl_puts $outchan "Content-type: text/html"
-			tcl_puts $outchan ""
 			tcl_puts $outchan "<html><head><title>Directory Listing Forbidden</title></head><body><h1>Directory Listing Forbidden</h1></body>"
 		} else {
 			# Return a 404 (File Not Found)
 			::rivet::cgi_server_writehttpheader 404 [array get env]
-			tcl_puts $outchan "Content-type: text/html"
-			tcl_puts $outchan ""
 			tcl_puts $outchan "<html><head><title>File Not Found</title></head><body><h1>File Not Found</h1></body>"
 		}
 
@@ -115,39 +103,31 @@ proc call_page {{useenv ""} {createinterp 0}} {
 			cd [file dirname $targetfile]
 
 			set env(SCRIPT_FILENAME) $targetfile
-			if {[info exists env(REQUEST_URI)]} {
-				set scriptfilenamework [lrange [file split $targetfile] 1 end]
-				set requesturiwork [lindex [split $env(REQUEST_URI) ?] 0]
-				set requesturiwork [split [string trim $requesturiwork /] /]
-				if {[llength $requesturiwork] == 0} {
-					set requesturiwork [list index.rvt]
-				}
-				for {set sfidx 0} {$sfidx < [llength $scriptfilenamework]} {incr sfidx} {
-					set endidx [expr $sfidx + [llength $requesturiwork] - 1]
-					set chklist [lrange $scriptfilenamework $sfidx $endidx]
-					if {$chklist == $requesturiwork} {
-						set env(SCRIPT_NAME) "/[join [lrange $scriptfilenamework $sfidx end] /]"
-						break
-					}
-				}
+
+			set topdir [string trimright $::starkit::topdir "/"]
+			set scriptname [string range $targetfile [string length $topdir] end]
+
+			if {[info exists env(SCRIPT_NAME)]} {
+				append env(SCRIPT_NAME) $scriptname
+			} else {
+				set env(SCRIPT_NAME) $scriptname
 			}
-	
+
 			if {$createinterp} {
 				set myinterp [interp create]
 
-				$myinterp eval [list set ::auto_path $::auto_path]
-				$myinterp eval [list package require tclrivet]
-				$myinterp eval [list unset -nocomplain ::env]
-				$myinterp eval [list array set ::env [array get env]]
-				$myinterp eval [list set ::rivet::parsestack [info script]]
-
-				foreach var [list ::starkit::topdir] {
+				foreach var [list ::starkit::topdir ::auto_path] {
 					if {[namespace qualifiers $var] != ""} {
 						$myinterp eval [list namespace eval [namespace qualifiers $var] ""]
 					}
 					$myinterp eval [list set $var [set $var]]
 				}
 
+				$myinterp eval [list package require tclrivet]
+				$myinterp eval [list unset -nocomplain ::env]
+				$myinterp eval [list array set ::env [array get env]]
+				$myinterp eval [list set ::rivet::parsestack [info script]]
+
 				if {$inchan != "stdin"} {
 					interp share {} $inchan $myinterp
 				}
@@ -162,15 +142,16 @@ proc call_page {{useenv ""} {createinterp 0}} {
 					$myinterp eval [list parse $targetfile]
 				} err]} {
 					$myinterp eval [list rivet_error]
-
-					$myinterp eval [list rivet_flush 1]
-
-					interp delete $myinterp
-					return
 				}
 
 				# Flush the output stream
-				$myinterp eval [list rivet_flush 1]
+				$myinterp eval [list rivet_flush -final]
+
+				# Determine result
+				set retval "close"
+				if {[$myinterp eval [list info exists ::rivet::connection]]} {
+					set retval [$myinterp eval [list set ::rivet::connection]]
+				}
 
 				interp delete $myinterp
 			} else {
@@ -185,17 +166,19 @@ proc call_page {{useenv ""} {createinterp 0}} {
 					parse $targetfile
 				} err]} {
 					rivet_error
-
-					rivet_flush 1
-
-					return
 				}
 
 				# Flush the output stream
-				rivet_flush 1
+				rivet_flush -final
+
+				# Determine result
+				set retval "close"
+				if {[info exists ::rivet::connection]} {
+					set retval $::rivet::connection
+				}
 			}
 	
-			return
+			return $retval
 		}
 		"*.ez" { set statictype "application/andrew-inset" }
 		"*.atom" { set statictype "application/atom+xml" }
@@ -706,25 +689,39 @@ proc call_page {{useenv ""} {createinterp 0}} {
 			set filelen [file size $targetfile]
 		}
 
+		headers type $statictype
+
+		catch {
+			headers set "Last-Modified" "[clock format [file mtime $targetfile] -format {%a, %d %b %Y %H:%M:%S GMT} -gmt 1]"
+
+			headers set "Expires" "Tue, 19 Jan 2038 03:14:07 GMT"
+		}
+
 		::rivet::cgi_server_writehttpheader 200 [array get env] $filelen
-		tcl_puts $outchan "Content-type: $statictype"
-		catch {
-			tcl_puts $outchan "Last-Modified: [clock format [file mtime $targetfile] -format {%a, %d %b %Y %H:%M:%S GMT} -gmt 1]"
-			tcl_puts $outchan "Expires: Tue, 19 Jan 2038 03:14:07 GMT"
-		}
-		tcl_puts $outchan ""
-	
-		set fd [open $targetfile r]
-		fconfigure $fd -encoding binary -translation {binary binary}
-		fconfigure $outchan -encoding binary -translation {binary binary}
 
-		# Do the copy in the foreground.
-		catch {
-			fcopy $fd $outchan
+		if {$filelen != "0"} {
+			set fd [open $targetfile r]
+			fconfigure $fd -encoding binary -translation {binary binary}
+			fconfigure $outchan -encoding binary -translation {binary binary}
+
+			# Do the copy in the foreground.
+			catch {
+				fcopy $fd $outchan
+			}
+
+			close $fd
 		}
 
-		close $fd
+		# Determine result
+		set retval "close"
+		if {[info exists ::rivet::connection]} {
+			set retval $::rivet::connection
+		}
+
+		return $retval
 	}
+
+	return "unknown"
 }
 
 proc print_help {} {
@@ -841,6 +838,7 @@ proc rivet_cgi_server {addr ports foreground initscp logfile errorlogfile maxthr
 			set logfd ""
 		}
 		default {
+			set logfile [file join [file dirname [file dirname [info script]]] $logfile]
 			set logfd [open $logfile a]
 		}
 	}
@@ -856,6 +854,7 @@ proc rivet_cgi_server {addr ports foreground initscp logfile errorlogfile maxthr
 			}
 		}
 		default {
+			set errorlogfile [file join [file dirname [file dirname [info script]]] $errorlogfile]
 			set elogfd [open $errorlogfile a]
 		}
 	}
@@ -890,9 +889,12 @@ proc rivet_cgi_server {addr ports foreground initscp logfile errorlogfile maxthr
 				set val $sslopts($opt)
 
 				switch -- $opt {
-					"require" {
+					"require" - "request" {
 						set val [expr $val]
 					}
+					"certfile" - "keyfile" - "cafile" - "cadir" {
+						set val [file normalize [file join [file dirname [file dirname [info script]]] $val]]
+					}
 					default {
 						if {$val == ""} {
 							continue
@@ -1106,9 +1108,9 @@ proc rivet_cgi_server_request {hostport logfd elogfd pmodel maxthreads httpmode
 
 		vwait ::rivetstarkit::finished($sock)
 
-		catch {
-			unset -nocomplain ::rivet_cgi_tls_verified($sock)
+		unset -nocomplain ::rivet_cgi_tls_verified($sock)
 
+		catch {
 			close $sock
 		}
 	} err]} {
@@ -1137,275 +1139,281 @@ proc rivet_cgi_server_request {hostport logfd elogfd pmodel maxthreads httpmode
 }
 
 proc rivet_cgi_server_request_data {sock addr hostport logfd elogfd pmodel} {
-	array set sockinfo $::rivetstarkit::sockinfo($sock)
+	if {[catch {
+		array set sockinfo $::rivetstarkit::sockinfo($sock)
 
-	gets $sock line
-	set line [string trimright $line "\r\n"]
+		gets $sock line
+		set line [string trimright $line "\r\n"]
 
-	if {$line == "" && [eof $sock]} {
-		# Tell the event loop that we're done here.
-		set ::rivetstarkit::finished($sock) 1
-
-		return
-	}
-
-	switch -- $sockinfo(state) {
-		"NEW" {
-			set sockinfo(requestline) $line
-			set work [split $line " "]
-
-			set sockinfo(method) [string toupper [lindex $work 0]]
-
-			if {[llength $work] == 2} {
-				set sockinfo(url) [lindex $work 1]
-				set sockinfo(httpproto) "HTTP/1.0"
-			} else {
-				set sockinfo(url) [join [lrange $work 1 end-1] " "]
-				set sockinfo(httpproto) [string toupper [lindex $work end]]
-			}
-
-			set sockinfo(state) HEADERS
-
-			set sockinfo(url) [regsub {http://[^/]*/} $sockinfo(url) {/}]
-
-			set work [split $sockinfo(url) ?]
-			set sockinfo(path) [lindex $work 0]
-			if {[llength $work] > 1} {
-				set sockinfo(query) [join [lrange $work 1 end] ?]
-			}
-
-			# We only support GET and POST, everyone else we just close on.
-			if {$sockinfo(method) != "GET" && $sockinfo(method) != "POST"} {
-				set ::rivetstarkit::finished($sock) 1
-
-				unset -nocomplain ::rivet_cgi_tls_verified($sock)
-
-				close $sock
-			}
-		}
-		"HEADERS" {
-			if {$line == ""} {
-				set sockinfo(state) HANDLEREQUEST
-			} else {
-				set work [split $line :]
-				set headervar [string toupper [lindex $work 0]]
-				set headerval [string trim [join [lrange $work 1 end] :]]
-				lappend sockinfo(headers) $headervar $headerval
-			}
-		}
-	}
-
-	if {$sockinfo(state) == "HANDLEREQUEST"} {
-		if {[info exists sockinfo(headers)]} {
-			array set headers $sockinfo(headers)
-		} else {
-			array set headers [list]
-		}
-
-		# Write log entry
-		if {[info exists headers(USER-AGENT)]} {
-			set ua $headers(USER-AGENT)
-		} else {
-			set ua "-"
-		}
-
-		fileevent $sock readable ""
-
-		set localinfo [fconfigure $sock -sockname]
-		if {![info exists headers(CONNECTION)]} {
-			switch -- $sockinfo(httpproto) {
-				"HTTP/1.1" {
-					set headers(CONNECTION) "keep-alive"
-				}
-				default {
-					set headers(CONNECTION) "close"
-				}
-			}
-		} else {
-			set headers(CONNECTION) [string tolower $headers(CONNECTION)]
-		}
-
-		if {![info exists headers(HOST)]} {
-			set headers(HOST) [lindex $localinfo 1]
-		}
-
-		# Create CGI/1.1 compatible environment
-		## Standard variables
-		set myenv(GATEWAY_INTERFACE) "CGI/1.1"
-		set myenv(SERVER_SOFTWARE) "Rivet Starkit"
-		set myenv(SERVER_NAME) [lindex [split $headers(HOST) :] 0]
-		set myenv(SERVER_PROTOCOL) $sockinfo(httpproto)
-		set myenv(SERVER_PORT) $hostport
-		set myenv(REQUEST_METHOD) $sockinfo(method)
-		set myenv(REMOTE_ADDR) $addr
-		set myenv(PATH_INFO) $sockinfo(path)
-		if {[info exists sockinfo(query)]} {
-			set myenv(QUERY_STRING) $sockinfo(query)
-		}
-		### Post requests have additional information
-		if {$sockinfo(method) == "POST"} {
-			if {[info exists headers(CONTENT-TYPE)]} {
-				set myenv(CONTENT_TYPE) $headers(CONTENT-TYPE)
-			}
-			if {[info exists headers(CONTENT-LENGTH)]} {
-				set myenv(CONTENT_LENGTH) $headers(CONTENT-LENGTH)
-			}
-		}
-		## Additional variables
-		set myenv(REQUEST_URI) $sockinfo(url)
-		set myenv(SERVER_ADDR) [lindex $localinfo 0]
-		set myenv(DOCUMENT_ROOT) $::starkit::topdir
-		if {[info exists headers(ACCEPT)]} {
-			set myenv(HTTP_ACCEPT) $headers(ACCEPT)
-		}
-		if {[info exists headers(USER-AGENT)]} {
-			set myenv(HTTP_USER_AGENT) $headers(USER-AGENT)
-		}
-		if {[info exists headers(REFERER)]} {
-			set myenv(HTTP_REFERER) $headers(REFERER)
-		}
-		## Cookies
-		if {[info exists headers(COOKIE)]} {
-			set myenv(HTTP_COOKIE) $headers(COOKIE)
-		}
-		## Copy some environment variables directly
-		if {[info exists ::env(SERVER_ADMIN)]} {
-			set myenv(SERVER_ADMIN) $::env(SERVER_ADMIN)
-		}
-		if {[info exists ::env(PATH)]} {
-			set myenv(PATH) $::env(PATH)
-		}
-
-		# Add Rivet Interface specification to fake environment, so further
-		# Rivet/CGI knows how to interface
-		set myenv(RIVET_INTERFACE) [list FULLHEADERS $sock $sock $elogfd [array get headers]]
-
-		# Set TLS Socket Info
-		array set tlsinfo_peer [list sbits 0]
-		array set tlsinfo_local [list sbits 0]
-		catch {
-			array set tlsinfo_peer [tls::status $sock]
-
-		}
-		catch {
-			array set tlsinfo_local [tls::status -local $sock]
-		}
-
-		## Set TLS client CGI Variables
-		if {$tlsinfo_peer(sbits) != 0} {
-			set myenv(HTTPS) on
-
-			if {![info exists ::rivet_cgi_tls_verified($sock)]} {
-				set ::rivet_cgi_tls_verified($sock) 0
-			}
-
-			if {$::rivet_cgi_tls_verified($sock) == "1"} {
-				set myenv(SSL_CLIENT_VERIFY) SUCCESS
-			} else {
-				unset -nocomplain myenv(SSL_CLIENT_VERIFY)
-			}
-
-			foreach {myenvvar tlsvar} [list SSL_CLIENT_S_DN subject SSL_CLIENT_I_DN issuer SSL_CLIENT_V_START notBefore SSL_CLIENT_V_END notAfter SSL_CLIENT_M_SERIAL serial SSL_CIPHER cipher SSL_CIPHER_USEKEYSIZE sbits] {
-				if {![info exists tlsinfo_peer($tlsvar)]} {
-					continue
-				}
-
-				set myenv($myenvvar) $tlsinfo_peer($tlsvar)
-			}
-		}
-
-		## Set TLS server CGI Variables
-		if {$tlsinfo_local(sbits) != 0} {
-			set myenv(HTTPS) on
-
-			foreach {myenvvar tlsvar} [list SSL_SERVER_S_DN subject SSL_SERVER_I_DN issuer SSL_SERVER_V_START notBefore SSL_SERVER_V_END notAfter SSL_SERVER_M_SERIAL serial SSL_CIPHER cipher SSL_CIPHER_USEKEYSIZE sbits] {
-				if {![info exists tlsinfo_local($tlsvar)]} {
-					continue
-				}
-
-				set myenv($myenvvar) $tlsinfo_local($tlsvar)
-			}
-		}
-
-		## Set TLS client/server X.509 component CGI variables
-		foreach locationvar [list tlsinfo_peer tlsinfo_local] {
-			switch -- $locationvar {
-				"tlsinfo_peer" {
-					set locationcgivar "CLIENT"
-				}
-				"tlsinfo_local" {
-					set locationcgivar "SERVER"
-				}
-			}
-
-			foreach type [list subject issuer] {
-				if {![info exists [set locationvar]($type)]} {
-					continue
-				}
-				switch -- $type {
-					"subject" {
-						set typecgivar "S"
-					}
-					"issuer" {
-						set typecgivar "I"
-					}
-				}
-
-				set curr_dn [set [set locationvar]($type)]
-				foreach component [split $curr_dn ,] {
-					set component_work [split $component =]
-					set component_name [string trim [string toupper [lindex $component_work 0]]]
-					set component_val [string trim [join [lrange $component_work 1 end] =]]
-
-					set myenv(SSL_${locationcgivar}_${typecgivar}_DN_${component_name}) $component_val
-				}
-			}
-		}
-
-		# Call "call_page" with the new enivronment
-		if {[catch {
-if {$elogfd != ""} {
-tcl_puts $elogfd "($sock/$addr/[pid]) Debug: [array get myenv] ++ headers = [array get headers]"
-flush $elogfd
-}
-
-			if {$pmodel == "flat"} {
-				call_page [array get myenv] 1
-			} else {
-				call_page [array get myenv] 0
-			}
-
-			if {$logfd != ""} {
-				tcl_puts $logfd "$addr - - \[[clock format [clock seconds] -format {%d/%b/%Y:%H:%M:%S %z}]\] \"$sockinfo(requestline)\" 200 0 \"-\" \"$ua\""
-				flush $logfd
-			}
-		} err]} {
-			if {$logfd != ""} {
-				tcl_puts $logfd "$addr - - \[[clock format [clock seconds] -format {%d/%b/%Y:%H:%M:%S %z}]\] \"$sockinfo(requestline)\" 500 0 \"-\" \"$ua\" \"Error: [join [split $err {"\n}]]\""
-				flush $logfd
-			}
-			if {$elogfd != ""} {
-				tcl_puts $elogfd "$err"
-				tcl_puts $elogfd "$::errorInfo"
-				flush $elogfd
-			}
-		}
-
-		# Cleanup
-		unset sockinfo
-		set sockinfo(state) NEW
-
-		if {$headers(CONNECTION) != "keep-alive"} {
+		if {$line == "" && [eof $sock]} {
 			# Tell the event loop that we're done here.
 			set ::rivetstarkit::finished($sock) 1
-		} else {
-			fconfigure $sock -buffering line
-			fconfigure $sock -translation crlf
-			fileevent $sock readable [list rivet_cgi_server_request_data $sock $addr $hostport $logfd $elogfd $pmodel]
+
+			return
+		}
+
+		switch -- $sockinfo(state) {
+			"NEW" {
+				set sockinfo(requestline) $line
+				set work [split $line " "]
+
+				set sockinfo(method) [string toupper [lindex $work 0]]
+
+				if {[llength $work] == 2} {
+					set sockinfo(url) [lindex $work 1]
+					set sockinfo(httpproto) "HTTP/1.0"
+				} else {
+					set sockinfo(url) [join [lrange $work 1 end-1] " "]
+					set sockinfo(httpproto) [string toupper [lindex $work end]]
+				}
+
+				set sockinfo(state) HEADERS
+
+				set sockinfo(url) [regsub {http://[^/]*/} $sockinfo(url) {/}]
+
+				set work [split $sockinfo(url) ?]
+				set sockinfo(path) [lindex $work 0]
+				if {[llength $work] > 1} {
+					set sockinfo(query) [join [lrange $work 1 end] ?]
+				}
+
+				# We only support GET and POST, everyone else we just close on.
+				if {$sockinfo(method) != "GET" && $sockinfo(method) != "POST"} {
+					set ::rivetstarkit::finished($sock) 1
+
+					unset -nocomplain ::rivet_cgi_tls_verified($sock)
+
+					close $sock
+				}
+			}
+			"HEADERS" {
+				if {$line == ""} {
+					set sockinfo(state) HANDLEREQUEST
+				} else {
+					set work [split $line :]
+					set headervar [string toupper [lindex $work 0]]
+					set headerval [string trim [join [lrange $work 1 end] :]]
+					lappend sockinfo(headers) $headervar $headerval
+				}
+			}
+		}
+
+		if {$sockinfo(state) == "HANDLEREQUEST"} {
+			if {[info exists sockinfo(headers)]} {
+				array set headers $sockinfo(headers)
+			} else {
+				array set headers [list]
+			}
+
+			# Write log entry
+			if {[info exists headers(USER-AGENT)]} {
+				set ua $headers(USER-AGENT)
+			} else {
+				set ua "-"
+			}
+
+			fileevent $sock readable ""
+
+			set localinfo [fconfigure $sock -sockname]
+			if {![info exists headers(CONNECTION)]} {
+				switch -- $sockinfo(httpproto) {
+					"HTTP/1.1" {
+						set headers(CONNECTION) "keep-alive"
+					}
+					default {
+						set headers(CONNECTION) "close"
+					}
+				}
+			} else {
+				set headers(CONNECTION) [string tolower $headers(CONNECTION)]
+			}
+
+			if {![info exists headers(HOST)]} {
+				set headers(HOST) [lindex $localinfo 1]
+			}
+
+			# Create CGI/1.1 compatible environment
+			## Standard variables
+			set myenv(GATEWAY_INTERFACE) "CGI/1.1"
+			set myenv(SERVER_SOFTWARE) "Rivet Starkit"
+			set myenv(SERVER_NAME) [lindex [split $headers(HOST) :] 0]
+			set myenv(SERVER_PROTOCOL) $sockinfo(httpproto)
+			set myenv(SERVER_PORT) $hostport
+			set myenv(REQUEST_METHOD) $sockinfo(method)
+			set myenv(REMOTE_ADDR) $addr
+			set myenv(PATH_INFO) $sockinfo(path)
+			if {[info exists sockinfo(query)]} {
+				set myenv(QUERY_STRING) $sockinfo(query)
+			}
+			### Post requests have additional information
+			if {$sockinfo(method) == "POST"} {
+				if {[info exists headers(CONTENT-TYPE)]} {
+					set myenv(CONTENT_TYPE) $headers(CONTENT-TYPE)
+				}
+				if {[info exists headers(CONTENT-LENGTH)]} {
+					set myenv(CONTENT_LENGTH) $headers(CONTENT-LENGTH)
+				}
+			}
+			## Additional variables
+			set myenv(REQUEST_URI) $sockinfo(url)
+			set myenv(SERVER_ADDR) [lindex $localinfo 0]
+			set myenv(DOCUMENT_ROOT) $::starkit::topdir
+			if {[info exists headers(ACCEPT)]} {
+				set myenv(HTTP_ACCEPT) $headers(ACCEPT)
+			}
+			if {[info exists headers(USER-AGENT)]} {
+				set myenv(HTTP_USER_AGENT) $headers(USER-AGENT)
+			}
+			if {[info exists headers(REFERER)]} {
+				set myenv(HTTP_REFERER) $headers(REFERER)
+			}
+			## Cookies
+			if {[info exists headers(COOKIE)]} {
+				set myenv(HTTP_COOKIE) $headers(COOKIE)
+			}
+			## Copy some environment variables directly
+			if {[info exists ::env(SERVER_ADMIN)]} {
+				set myenv(SERVER_ADMIN) $::env(SERVER_ADMIN)
+			}
+			if {[info exists ::env(PATH)]} {
+				set myenv(PATH) $::env(PATH)
+			}
+
+			# Add Rivet Interface specification to fake environment, so further
+			# Rivet/CGI knows how to interface
+			set myenv(RIVET_INTERFACE) [list FULLHEADERS $sock $sock $elogfd [array get headers]]
+
+			# Set TLS Socket Info
+			array set tlsinfo_peer [list sbits 0]
+			array set tlsinfo_local [list sbits 0]
+			catch {
+				array set tlsinfo_peer [tls::status $sock]
+
+			}
+			catch {
+				array set tlsinfo_local [tls::status -local $sock]
+			}
+
+			## Set TLS client CGI Variables
+			if {$tlsinfo_peer(sbits) != 0} {
+				set myenv(HTTPS) on
+
+				if {![info exists ::rivet_cgi_tls_verified($sock)]} {
+					set ::rivet_cgi_tls_verified($sock) 0
+				}
+
+				if {$::rivet_cgi_tls_verified($sock) == "1"} {
+					set myenv(SSL_CLIENT_VERIFY) SUCCESS
+				} else {
+					unset -nocomplain myenv(SSL_CLIENT_VERIFY)
+				}
+
+				foreach {myenvvar tlsvar} [list SSL_CLIENT_S_DN subject SSL_CLIENT_I_DN issuer SSL_CLIENT_V_START notBefore SSL_CLIENT_V_END notAfter SSL_CLIENT_M_SERIAL serial SSL_CIPHER cipher SSL_CIPHER_USEKEYSIZE sbits] {
+					if {![info exists tlsinfo_peer($tlsvar)]} {
+						continue
+					}
+
+					set myenv($myenvvar) $tlsinfo_peer($tlsvar)
+				}
+			}
+
+			## Set TLS server CGI Variables
+			if {$tlsinfo_local(sbits) != 0} {
+				set myenv(HTTPS) on
+
+				foreach {myenvvar tlsvar} [list SSL_SERVER_S_DN subject SSL_SERVER_I_DN issuer SSL_SERVER_V_START notBefore SSL_SERVER_V_END notAfter SSL_SERVER_M_SERIAL serial SSL_CIPHER cipher SSL_CIPHER_USEKEYSIZE sbits] {
+					if {![info exists tlsinfo_local($tlsvar)]} {
+						continue
+					}
+
+					set myenv($myenvvar) $tlsinfo_local($tlsvar)
+				}
+			}
+
+			## Set TLS client/server X.509 component CGI variables
+			foreach locationvar [list tlsinfo_peer tlsinfo_local] {
+				switch -- $locationvar {
+					"tlsinfo_peer" {
+						set locationcgivar "CLIENT"
+					}
+					"tlsinfo_local" {
+						set locationcgivar "SERVER"
+					}
+				}
+
+				foreach type [list subject issuer] {
+					if {![info exists [set locationvar]($type)]} {
+						continue
+					}
+					switch -- $type {
+						"subject" {
+							set typecgivar "S"
+						}
+						"issuer" {
+							set typecgivar "I"
+						}
+					}
+
+					set curr_dn [set [set locationvar]($type)]
+					foreach component [split $curr_dn ,] {
+						set component_work [split $component =]
+						set component_name [string trim [string toupper [lindex $component_work 0]]]
+						set component_val [string trim [join [lrange $component_work 1 end] =]]
+
+						set myenv(SSL_${locationcgivar}_${typecgivar}_DN_${component_name}) $component_val
+					}
+				}
+			}
+
+			# Call "call_page" with the new enivronment
+			set result "close"
+			if {[catch {
+				if {$pmodel == "flat"} {
+					set result [call_page [array get myenv] 1]
+				} else {
+					set result [call_page [array get myenv] 0]
+				}
+
+				if {$logfd != ""} {
+					tcl_puts $logfd "$addr - - \[[clock format [clock seconds] -format {%d/%b/%Y:%H:%M:%S %z}]\] \"$sockinfo(requestline)\" 200 0 \"-\" \"$ua\""
+					flush $logfd
+				}
+			} err]} {
+				if {$logfd != ""} {
+					tcl_puts $logfd "$addr - - \[[clock format [clock seconds] -format {%d/%b/%Y:%H:%M:%S %z}]\] \"$sockinfo(requestline)\" 500 0 \"-\" \"$ua\" \"Error: [join [split $err {"\n}]]\""
+					flush $logfd
+				}
+
+				if {$elogfd != ""} {
+					tcl_puts $elogfd "($sock/$addr/[pid]) $err"
+					tcl_puts $elogfd "($sock/$addr/[pid]) $::errorInfo"
+					flush $elogfd
+				}
+			}
+
+			# Cleanup
+			unset sockinfo
+			set sockinfo(state) NEW
+
+			if {$headers(CONNECTION) != "keep-alive" || $result != "keep-alive"} {
+				# Tell the event loop that we're done here.
+				set ::rivetstarkit::finished($sock) 1
+			} else {
+				fconfigure $sock -buffering line
+				fconfigure $sock -translation crlf
+				fileevent $sock readable [list rivet_cgi_server_request_data $sock $addr $hostport $logfd $elogfd $pmodel]
+			}
+		}
+ 
+		set ::rivetstarkit::sockinfo($sock) [array get sockinfo]
+	}] == "1"} {
+		# In case of error, abort.
+		set ::rivetstarkit::finished($sock) 1
+		if {$elogfd != ""} {
+			tcl_puts $elogfd "($sock/$addr/[pid]) $::errorInfo"
+			flush $elogfd
 		}
 	}
- 
-	set ::rivetstarkit::sockinfo($sock) [array get sockinfo]
 }
 
 
@@ -1416,6 +1424,21 @@ if {![info exists ::env(GATEWAY_INTERFACE)]} {
 
 	switch -- $cmd {
 		"--server" {
+			set conffile [file dirname [info script]]
+			append conffile ".conf"
+			if {[file exists $conffile]} {
+				set argv_conf [list]
+				catch {
+					set fd [open $conffile]
+
+					set argv_conf [read -nonewline $fd]
+
+					close $fd
+				}
+
+				set argv [concat $argv_conf $argv]
+			}
+
 			set options(--address) "ALL"
 			set options(--port) 80
 			set options(--foreground) no
@@ -1457,7 +1480,7 @@ if {![info exists ::env(GATEWAY_INTERFACE)]} {
 			exit 0
 		}
 		"--version" {
-			tcl_puts "RivetStarkit version 0.5.0.2634"
+			tcl_puts "RivetStarkit version 0.5.0.2664"
 			exit 0
 		}
 		default {
